@@ -18,6 +18,7 @@ module Yesod.Json
 
       -- * Convenience functions
     , jsonOrRedirect
+    , acceptsJson
     ) where
 
 import Yesod.Handler (GHandler, waiRequest, lift, invalidArgs, redirect)
@@ -32,6 +33,7 @@ import Control.Arrow (second)
 import Control.Applicative ((<$>))
 import Control.Monad (join)
 import qualified Data.Aeson as J
+import qualified Data.Aeson.Parser as JP
 import Data.Aeson ((.=))
 import qualified Data.Aeson.Encode as JE
 import Data.Aeson.Encode (fromValue)
@@ -49,11 +51,13 @@ import Network.Wai.Parse (parseHttpAccept)
 import qualified Data.ByteString.Char8 as B8
 import Safe (headMay)
 
+#if !MIN_VERSION_yesod_core(1, 1, 5)
 instance ToContent J.Value where
     toContent = flip ContentBuilder Nothing
               . Blaze.fromLazyText
               . toLazyText
               . fromValue
+#endif
 
 -- | Provide both an HTML and JSON representation for a piece of
 -- data, using the default layout for the HTML output
@@ -84,7 +88,7 @@ jsonToRepJson = return . RepJson . toContent . J.toJSON
 parseJsonBody :: J.FromJSON a => GHandler sub master (J.Result a)
 parseJsonBody = do
     req <- waiRequest
-    fmap J.fromJSON $ lift $ requestBody req $$ sinkParser J.json'
+    fmap J.fromJSON $ lift $ requestBody req $$ sinkParser JP.value'
 
 -- | Same as 'parseJsonBody', but return an invalid args response on a parse
 -- error.
@@ -95,8 +99,10 @@ parseJsonBody_ = do
         J.Error s -> invalidArgs [pack s]
         J.Success a -> return a
 
+#if !MIN_VERSION_shakespeare_js(1, 0, 2)
 instance ToJavascript J.Value where
     toJavascript = fromLazyText . decodeUtf8 . JE.encode
+#endif
 
 -- | Convert a list of pairs to an 'J.Object'.
 object :: J.ToJSON a => [(Text, a)] -> J.Value
@@ -109,7 +115,8 @@ array = J.Array . V.fromList . map J.toJSON
 -- | jsonOrRedirect simplifies the scenario where a POST handler sends a different
 -- response based on Accept headers:
 --
---     1. 200 with JSON data if the client prefers application/json (e.g. AJAX).
+--     1. 200 with JSON data if the client prefers
+--     @application\/json@ (e.g. AJAX, see 'acceptsJSON').
 --
 --     2. 3xx otherwise, following the PRG pattern.
 jsonOrRedirect :: (Yesod master, J.ToJSON a)
@@ -120,9 +127,12 @@ jsonOrRedirect r j = do
     q <- acceptsJson
     if q then jsonToRepJson (J.toJSON j)
          else redirect r
-  where
-    acceptsJson = maybe False ((== "application/json") . B8.takeWhile (/= ';'))
-                . join
-                . fmap (headMay . parseHttpAccept)
-                . lookup "Accept" . requestHeaders
-                <$> waiRequest
+
+-- | Returns @True@ if the client prefers @application\/json@ as
+-- indicated by the @Accept@ HTTP header.
+acceptsJson :: Yesod master => GHandler sub master Bool
+acceptsJson =  maybe False ((== "application/json") . B8.takeWhile (/= ';'))
+            .  join
+            .  fmap (headMay . parseHttpAccept)
+            .  lookup "Accept" . requestHeaders
+           <$> waiRequest
